>From bac195b2c359b3dd311e30d7f17f83d09ece4438 Mon Sep 17 00:00:00 2001
From: Andres Freund <andres@anarazel.de>
Date: Fri, 10 Oct 2014 13:03:56 +0200
Subject: [PATCH 4/4] Spelling and other comment improvements for the new
 lwlock code.

Robert Haas
---
 src/backend/storage/lmgr/lwlock.c | 50 ++++++++++++++++++++++-----------------
 1 file changed, 28 insertions(+), 22 deletions(-)

diff --git a/src/backend/storage/lmgr/lwlock.c b/src/backend/storage/lmgr/lwlock.c
index 2246fc3..d1f0f77 100644
--- a/src/backend/storage/lmgr/lwlock.c
+++ b/src/backend/storage/lmgr/lwlock.c
@@ -29,40 +29,46 @@
  * This used to be a pretty straight forward reader-writer lock
  * implementation, in which the internal state was protected by a
  * spinlock. Unfortunately the overhead of taking the spinlock proved to be
- * too high for workloads/locks that were locked in shared mode very
- * frequently. Often we were spinning in the (obviously exlusive) spinlock,
+ * too high for workloads/locks that were taken in shared mode very
+ * frequently. Often we were spinning in the (obviously exclusive) spinlock,
  * while trying to acquire a shared lock that was actually free.
  *
  * Thus a new implementation was devised that provides wait-free shared lock
- * acquiration for locks that aren't exclusively locked.
+ * acquisition for locks that aren't exclusively locked.
  *
  * The basic idea is to have a single atomic variable 'lockcount' instead of
  * the formerly separate shared and exclusive counters and to use an atomic
  * increment to acquire the lock. That's fairly easy to do for rw-spinlocks,
  * but a lot harder for something like LWLocks that want to wait in the OS.
  *
- * For exlusive lock acquisition we use an atomic compare-and-exchange on the
- * lockcount variable swapping in EXCLUSIVE_LOCK/1<<31-1/0x7FFFFFFF if and only
- * if the current value of lockcount is 0. If the swap was not successfull, we
- * have to wait.
+ * For exclusive lock acquisition we use an atomic compare-and-exchange on the
+ * lockcount variable, swapping in a sentinel value
+ * (EXCLUSIVE_LOCK/1<<31-1/0x7FFFFFFF) if and only if the current value of
+ * lockcount is 0. If the swap was not successful, we have to wait.
  *
  * For shared lock acquisition we use an atomic add to the lockcount variable
- * to add 1. If the value is bigger than EXCLUSIVE_LOCK we know that somebody
+ * to add 1. If the value returned by the atomic fetch-and-add operation is
+ * bigger than the value marking an exclusive locks we know that somebody else
  * actually has an exclusive lock, and we back out by atomically decrementing
- * by 1 again. If so, we have to wait for the exlusive locker to release the
+ * by 1 again. If so, we have to wait for the exclusive locker to release the
  * lock.
  *
  * To release the lock we use an atomic decrement to release the lock. If the
  * new value is zero (we get that atomically), we know we have to release
  * waiters.
  *
- * The attentive reader probably might have noticed that naively doing the
- * above has two glaring race conditions:
+ * Obviously it is important that the sentinel value for exclusive locks
+ * doesn't conflict with the maximum number of possible share lockers -
+ * luckily MAX_BACKENDS makes that easily possible.
+ *
+ *
+ * The attentive reader might have noticed that naively doing the above has
+ * two glaring race conditions:
  *
  * 1) too-quick-for-queueing: We try to lock using the atomic operations and
- * notice that we have to wait. Unfortunately until we have finished queuing,
- * the former locker very well might have already finished it's work. That's
- * problematic because we're now stuck waiting inside the OS.
+ * notice that we have to wait. Unfortunately by the time we have finished
+ * queuing, the former locker very well might have already finished it's
+ * work. That's problematic because we're now stuck waiting inside the OS.
  *
  * 2) spurious failed locks: Due to the logic of backing out of shared
  * locks after we unconditionally added a 1 to lockcount, we might have
@@ -80,23 +86,23 @@
  *
  * To mitigate those races we use a two phased attempt at locking:
  *   Phase 1: Try to do it atomically, if we succeed, nice
- *   Phase 2: Add us too the waitqueue of the lock
+ *   Phase 2: Add ourselves to the waitqueue of the lock
  *   Phase 3: Try to grab the lock again, if we succeed, remove ourselves from
  *            the queue
- *   Phase 4: Sleep till wakeup, goto Phase 1
+ *   Phase 4: Sleep till wake-up, goto Phase 1
  *
  * This protects us against both problems from above:
  * 1) Nobody can release too quick, before we're queued, since after Phase 2
  *    we're already queued.
  * 2) If somebody spuriously got blocked from acquiring the lock, they will
- *    get queued in Phase 2 and we can wake them up if neccessary or they will
+ *    get queued in Phase 2 and we can wake them up if necessary or they will
  *    have gotten the lock in Phase 3.
  *
  * There above algorithm only works for LWLockAcquire, not directly for
  * LWLockAcquireConditional where we don't want to wait. In that case we just
- * need to retry acquiring the lock until we're sure we didn't disturb anybody
- * in doing so.
- *-------------------------------------------------------------------------
+ * need to retry acquiring the lock until we're sure we didn't spuriously
+ * prevent anybody else from doing so.
+ * -------------------------------------------------------------------------
  */
 #include "postgres.h"
 
@@ -631,7 +637,7 @@ LWLockAttemptLock(LWLock* lock, LWLockMode mode, bool *potentially_spurious)
 		}
 		else
 		{
-			/* yipeyyahee */
+			/* Great! Got the lock. */
 #ifdef LOCK_DEBUG
 			lock->owner = MyProc;
 #endif
@@ -687,7 +693,7 @@ LWLockAttemptLock(LWLock* lock, LWLockMode mode, bool *potentially_spurious)
 		}
 		else
 		{
-			/* yipeyyahee */
+			/* Great! Got the lock. */
 			return false;
 		}
 	}
-- 
1.8.3.251.g1462b67

